//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// IBChess
// Copyright (C) 2002 by Ihor Bobak, ibobak@ukrpost.net
//------------------------------------------------------------------------------
// The project consists of 9 modules:
// global.pas: contains all constants and global variables used by other modules
// move.pas: the class of move
// board.pas: the class of board
// boardeval.pas: estimation function
// movehenerator.pas: move generator
// tables.pas: transposition table and history heuristics
// OpeningBook.pas: opening book
// SearchAgent.pas: the main program module which containt she search algorithm
// Player.pas: the classes of players (human and PC)
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

{$APPTYPE CONSOLE}
program ibchess;

uses
  SysUtils,
  global in 'global.pas',
  move in 'move.pas',
  board in 'Board.pas',
  boardeval in 'boardeval.pas',
  movegenerator in 'movegenerator.pas',
  tables in 'tables.pas',
  OpeningBook in 'OpeningBook.pas',
  SearchAgent in 'SearchAgent.pas',
  Player in 'Player.pas';

{$R *.RES}

var
  Openings: TOpeningBook;
  GameBoard: TBoard;
  Players: array[0..1] of TPlayer;


//******************************************************************************
// Initialization of objects, necessary for the game
//******************************************************************************
procedure InitializeGame(openingBook: string);
var
  Key: string;
begin
  // create the opening book database
  Openings := TOpeningBook.Create;
  Openings.Load(openingBook);
  // create the chess board
  GameBoard := TBoard.Create;
  GameBoard.InitializeBoard;
  // define the players and create the instances of the corresponfing
  // classes (TPlayerAI or TPlayerHuman)
  key := 'C';
  write('Who plays white: [h]uman or [c]omputer? ');
  repeat
    readln(key);
    if (length(key)>0) and
      ((key[1] = 'H') or (key[1] = 'h') or (key = 'c') or (key = 'C')) then
      break;
  until false;

  if ((key[1] = 'H') or (key[1] = 'h')) then
    Players[ SIDE_WHITE ] := TPlayerHuman.Create(SIDE_WHITE)
  else
    Players[ SIDE_WHITE ] := TPlayerAI.Create(SIDE_WHITE, Openings);

  write('Who plays black: [h]uman or [c]omputer? ');
  repeat
    readln(key);
    if (length(key)>0) and
      ((key[1] = 'H') or (key[1] = 'h') or (key = 'c') or (key = 'C')) then
      break;
  until false;

  if ((key[1] = 'H') or (key[1] = 'h')) then
    Players[ SIDE_BLACK ] := TPlayerHuman.Create(SIDE_BLACK)
  else
    Players[ SIDE_BLACK ] :=
      TPlayerAI.Create(SIDE_BLACK, Openings);
end;

//******************************************************************************
// The main cycle of the game
//******************************************************************************
procedure RunGame;
var
  CurrentPlayer: TPlayer;
  Mov: TMove;
begin
  Mov := nil;
  // this cycle sequentially asks moves from the players;
  // if the player is PC (instance of TPlayerAI),
  // the move will be selected by teh algorithm; if the move is
  // done by a human player, it is entered from the keyboard.
  // All this is done with the help of the vitrual function GetMove
  repeat
    GameBoard.Print;
    CurrentPlayer := Players[ GameBoard.CurrentPlayer ];
    if Mov<>nil then Mov.Free;
    Mov := CurrentPlayer.GetMove(GameBoard);
    write(PlayerStrings[ GameBoard.CurrentPlayer ], ' selects move: '); Mov.Println;
    GameBoard.ApplyMove(Mov);
  until not
    ((Mov.MoveType <> MOVE_RESIGN) and (Mov.MoveType <> MOVE_STALEMATE));

  writeln('Game Over.  Thanks for playing!') ;
end;


//******************************************************************************
// Disposing of the memory
//******************************************************************************
procedure FinalizeGame;
begin
  GameBoard.Free;
  Players[0].Free;
  Players[1].Free;
  Openings.Free;
end;


begin
  InitializeGame('openings.txt');
  RunGame;
  FinalizeGame;
  readln;
end.
