//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// IBChess
// Copyright (C) 2002 by Ihor Bobak, ibobak@ukrpost.net
//------------------------------------------------------------------------------
// move class
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
unit move;

interface

uses global;

type
  TMove = class
  public
    MovingPiece: integer; // moving piece
    CapturedPiece: integer; // captured piece
    SourceSquare: integer; // "from" square
    DestinationSquare: integer; // "to" square
    MoveType: integer; // move type (see constants in global.pas)
  public
    constructor Create;
    procedure Clone(target: TMove);
    function Equals(target: TMove): boolean;
    function Reset: boolean;
    procedure Print;
    procedure Println;
    function ToString: string;
  end;

implementation

constructor TMove.Create;
begin
  self.Reset;
end;

procedure TMove.Clone(target: TMove);
begin
  MovingPiece        := target.MovingPiece;
  CapturedPiece      := target.CapturedPiece;
  SourceSquare       := target.SourceSquare;
  DestinationSquare  := target.DestinationSquare;
  MoveType           := target.MoveType;
end;

function TMove.Equals(target: TMove): boolean;
begin
  Result := false;
  if MovingPiece <> target.MovingPiece then exit;
  if CapturedPiece <> target.CapturedPiece then exit;
  if MoveType <> target.MoveType then exit;
  if SourceSquare <> target.SourceSquare then exit;
  if DestinationSquare <> target.DestinationSquare then exit;
  Result := true;
end;

function TMove.Reset: boolean;
begin
  MovingPiece := EMPTY_SQUARE;
  CapturedPiece := EMPTY_SQUARE;
  SourceSquare := NULL_MOVE;
  DestinationSquare := NULL_MOVE;
  MoveType := NULL_MOVE;
  Result := true;
end;

procedure TMove.Print;
begin
  write(ToString);
end;

procedure TMove.Println;
begin
  writeln(ToString);
end;


function TMove.ToString: string;
begin
  if MoveType = MOVE_STALEMATE then
    Result := 'STALEMATE!!!';
  if MoveType <> MOVE_RESIGN then
    Result := PieceStrings[ MovingPiece ] +' '+ SquareToNotation(SourceSquare) + '-'+ SquareToNotation(DestinationSquare)
  else
    Result := 'RESIGNATION!';
end;

end.
